class Utils {
    /**
     * 发送Get请求
     *
     * @param {!string} url 请求地址
     * @param {?function} next 回调函数
     */
    static ajaxGet(url, next) {
        let xhr = new XMLHttpRequest();

        if (url.includes('?')) {
            // 在URL有其他参数时，添加一个date参数加入当前时间以避免缓存
            xhr.open('GET', `${url}&date=${new Date().getTime()}`, true);
        } else {
            // 添加一个date参数加入当前时间以避免缓存
            xhr.open('GET', `${url}&date=${new Date().getTime()}`, true);
        }
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        // 更改XMLHttpRequest对象withCredentials属性以支持跨域Cookies
        xhr.withCredentials = true;
        xhr.responseType = 'json';
        xhr.onload = function (res) {
            // 获取请求接口返回值
            let response = res.target.response;

            next && next(response);
        };
        xhr.send();
    }
    /**
     * 发送Post请求
     *
     * @param {!string} url 请求地址
     * @param {?string} data post请求参数
     * @param {?function} next 回调函数
     */
    static ajaxPost(url, data, next) {
        let xhr = new XMLHttpRequest();

        if (url.includes('?')) {
            // 在URL有其他参数时，添加一个date参数加入当前时间以避免缓存
            xhr.open("POST", `${url}&${new Date().getTime()}`, true);
        } else {
            // 添加一个date参数加入当前时间以避免缓存
            xhr.open("POST", `${url}?${new Date().getTime()}`, true);
        }
        xhr.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        // 更改XMLHttpRequest对象withCredentials属性以支持跨域Cookies
        xhr.withCredentials = true;
        xhr.responseType = 'json';
        xhr.onload = function (res) {
            // 获取请求接口返回值
            let response = res.target.response;

            next && next(response);
        };
        xhr.send(data);
    }
}

window.onload = function(){
    let signupButton = document.querySelector('#signup-button')
    signupButton.addEventListener('click',() => {
        let userId = document.querySelector('input[name=userId]').value
        let password = document.querySelector('input[name=password]').value
        let iCode = document.querySelector('input[name=iCode]').value
        let cCode = document.querySelector('input[name=cCode]').value

        let signUpUrl = 'http://admin.cloudsql.1473.cn/v1/signup'
        let postData = `userId=${userId}&password=${password}&iCode=${iCode}&captcha=${cCode}`

        Utils.ajaxPost(signUpUrl, postData, res => {
            if(res.status == 'successed'){
                // mdui.snckbar({
                //     message: response.info
                // })
                alert(response.info)
                window.location.hash = response.url
            }else{
                alert("注册失败");
            }
        })
    })
}
